#ifndef _TPM_LOGIN_H_
#define _TPM_LOGIN_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <dlfcn.h>
#include <errno.h>

#define TPM_NAME_LEN 128
#define TPM_INFO_LEN 128
#define TPM_LOGIN_LEN 128
#define TPM_AUTHTOK_LEN 128

typedef struct tpm_login_plugin
{
    char* name;
    void* handle;
    int (*pfn_init)(void);
    int (*pfn_read_iuim)(void);
    int (*pfn_user_login)(char *username, int len, int *autologon);
    int (*pfn_check_iuim)(void);
    int (*pfn_get_authtok)(char *curauthtok, int curlen, char *oldauthtok, int oldlen, long int *chdate);
    const char *(*pfn_strerror)(int error);
    const char *(*pfn_get_info)(void);
} tpm_login_plugin_t;

#define TPM_LOAD(tpm, plugin) \
    do{ \
        plugin.name = strdup(tpm); \
        plugin.handle = dlopen(plugin.name, RTLD_LAZY); \
        if(plugin.handle){ \
            plugin.pfn_init = dlsym(plugin.handle, "tpm_init"); \
            plugin.pfn_read_iuim = dlsym(plugin.handle, "tpm_read_iuim"); \
            plugin.pfn_user_login = dlsym(plugin.handle, "tpm_user_login"); \
            plugin.pfn_check_iuim = dlsym(plugin.handle, "tpm_check_iuim"); \
            plugin.pfn_get_authtok = dlsym(plugin.handle, "tpm_get_authtok"); \
            plugin.pfn_strerror = dlsym(plugin.handle, "tpm_strerror"); \
            plugin.pfn_get_info = dlsym(plugin.handle, "tpm_get_info"); \
            if(!plugin.pfn_init \
                || !plugin.pfn_read_iuim \
                || !plugin.pfn_user_login \
                || !plugin.pfn_check_iuim \
                || !plugin.pfn_get_authtok \
                || !plugin.pfn_strerror \
                || !plugin.pfn_get_info){ \
                dlclose(plugin.handle); \
                if(plugin.name) \
                    free(plugin.name); \
                memset(&plugin, 0, sizeof(struct tpm_login_plugin)); \
            } \
        } \
    }while(0);

#define TPM_LOADED(plugin) (plugin.handle != NULL)

#define TPM_FREE(plugin) \
    do{ \
        if(plugin.handle) \
            dlclose(plugin.handle); \
        if(plugin.name) \
            free(plugin.name); \
        memset(&plugin, 0, sizeof(struct tpm_login_plugin)); \
    }while(0);

#endif // _TPM_LOGIN_H_
